@@ -1,15 +1,20 @@
 import os
-from gateoverflow import state as s
+from gateoverflow.state import state as s
 from gateoverflow import constants
 from gateoverflow.logger import d
 from gateoverflow import queries as q
-from gateoverflow.helpers import readable_date, open_link, uncrawled_metadata_count, crawl_metadata, prettify_table, print_logo
+from gateoverflow.helpers import readable_date, open_link, uncrawled_metadata_count, crawl_metadata, prettify_table, print_logo, ask
 modes = constants.modes
 
 
+def abort_program(status=1):
+    print("Abort.")
+    exit(int(status))
+
+
 def exit_program():
     print("Okay, bye.")
-    s.stop = True
+    s['stop'] = True
 
 
 def print_help():
@@ -29,11 +34,11 @@ def clear_screen():
 
 def debug_toggle():
     res = ''
-    if s.DEBUG == True:
-        s.DEBUG = False
+    if s['DEBUG'] == True:
+        s['DEBUG'] = False
         res = 'OFF'
     else:
-        s.DEBUG = True
+        s['DEBUG'] = True
         res = 'ON'
     print(f'debug mode is {res}')
 
@@ -50,14 +55,14 @@ def get_switcher():
     switchers = {
         modes.DEFAULT: switcher,
     }
-    return switchers[s.mode]
+    return switchers[s['mode']]
 
 
 def list_command():
     # list_options = set(['r', 'q', 't', 'recent', 'questions', 'tags'])
     row_offset = 0
-    cmd = s.list_string.split(' ')
-    c = s.cursor
+    cmd = s['list_string'].split(' ')
+    c = s['cursor']
     d(print, 'lists recents')
     d(print, f'probably, you wanted to list: {cmd}')
     if len(cmd) > 2:
@@ -68,26 +73,43 @@ def list_command():
     try:
         how_many = cmd[1]
     except:
-        how_many = s.how_many  # default show only 10 records?
+        how_many = s['how_many']  # default show only 10 records?
     # what_to_show = cmd[1]
     d(print, f"omg you want this? : recents, {how_many} items")
     c.execute(q.get_recent, (how_many, row_offset))
     res = c.fetchall()
     # title = 'QuestionID'.ljust(12) + 'Visited'.ljust(12) + 'Time'.ljust(12)
     # print(title)
-    headers = ['Question ID', 'Visited', 'Time']
+    headers = ['Question ID', 'Title', 'Description', 'Visits', 'Last Visited']
     data = []
+    k = s['column_width']
     for row in res:
         row = [str(each) for each in row]
-        data.append([row[0], row[1], readable_date(row[2])])
+        frow = []
+        for each in row:
+            temp = ''
+            if(each == 'None'):
+                temp = 'NA'
+            if(len(each) > k):
+                d(print, f'length: {len(each)}')
+                each = each.replace("- GATE Overflow", "")
+                temp = each
+                if(len(each) > k):
+                    temp = each[:k-3] + "..."
+            else:
+                temp = each
+            frow.append(temp)
+        for each in frow:
+            d(print, f'after len: {len(each)}')
+        data.append([frow[0], frow[1], frow[2],
+                     frow[3], readable_date(frow[4])])
     print(prettify_table(data, headers))
 
 
 def crawler():
     count = uncrawled_metadata_count()
     print(f'Unscraped records: {count}')
-    q = input("Do you want to continue?(yes/no) (default: no): ")
-    if(q.lower() == 'yes' or q.lower() == 'y'):
+    if(ask()):
         # continue
         crawl_metadata()
     else:
@@ -102,21 +124,22 @@ def invalid_command():
 class Parser:
     @staticmethod
     def list_tags():
-        c = s.cursor
+        c = s['cursor']
         res = c.execute(q.get_tags)
         print(f'List of tags...')
         if res == None:
             print("No tags are created yet :(")
         for each in res:
-            print(f'#{each[0]} ({each[1]} questions)')
+            print(
+                f'{constants.colors.BLUE}#{each[0]}{constants.colors.END} ( {constants.colors.FAIL}{each[1]}{constants.colors.END} questions)')
 
     @staticmethod
     def open_questions():
         print('Number(s) found, treating them as questionIDs, and opening each in web-browser...')
         # get the input list, stored in state
-        question_ids = s.questions_list
+        question_ids = s['questions_list']
         # insert into recent questions table, along with timestamp, update visited count if already visited
-        c = s.cursor
+        c = s['cursor']
         for each in question_ids:
             # check if already exists
             c.execute(q.get_question, [each])
@@ -129,14 +152,15 @@ class Parser:
                 # insert
                 c.execute(q.insert_into_recents, [each])
             open_link(f'https://gateoverflow.in/{each}')
-        s.conn.commit()
+        s['conn'].commit()
         print('Done!')
 
     @staticmethod
     def list_questions_of_tags():
         print(f'Only tag(s) found, listing questions of specified tags...')
-        tags, questions = s.tags, s.questions_list
-        c = s.cursor
+        d('t', 'listing questions for tags')
+        tags, questions = s['tags'], s['questions_list']
+        c = s['cursor']
         res = c.execute(q.get_tags)
         tags_in_db = []
         for each in res:
@@ -146,26 +170,37 @@ class Parser:
         tags = [a[1:] for a in tags]
         for each in tags:
             if each not in tags_in_db:
-                d(print, f'Tag {each} does not exist.')
+                print(f'#{each} does not exist.')
                 non_existant_tags.append(each)
                 continue
             existant_tags.append(each)
         d(print, f'Existant tags: {existant_tags}')
         if len(existant_tags) != 0:
             # TODO: find a way to move this query to queries.py
-            query = f'SELECT question_id, (select name from tags where id=tag_id) from tq_relation as TQ WHERE tag_id IN (SELECT id FROM tags WHERE name IN ({"?"*len(existant_tags)}))'
+            bruh = (",?"*len(existant_tags)).split(',')[1:]
+            bruh = ','.join(bruh)
+            query = f'select ogq as question_id, tagname, title, visited_count desc from (select question_id as ogq, (select tags.name from tags where id=tag_id)as tagname from tq_relation where tag_id in (select id from tags where name in ({bruh}))) left join metadata on ogq = metadata.question_id left join recents on ogq = recents.question_id;'
+            d(print, query)
             res = c.execute(query, existant_tags)
             if res == None:
                 print('Nothing to show.')
                 return
-            for each in res:
-                d(print, f'Question:{each[0]}\t Tag: #{each[1]}')
+            headers = ['QuestionID', 'Tag', 'Title',
+                       'Description', 'Visited Count']
+            k = s['column_width']
+            data = []
+            for row in res:
+                row = [str(each) for each in row]
+                data.append(row)
+            print(prettify_table(data, headers))
+        else:
+            print("Nothing to show.")
 
     @staticmethod
     def add_questions_to_tags():
-        print(f'Adding {s.questions_list} to {s.tags}...')
-        tags, questions = s.tags, s.questions_list
-        c = s.cursor
+        print(f'Adding {s["questions_list"]} to {s["tags"]}...')
+        tags, questions = s['tags'], s['questions_list']
+        c = s['cursor']
         res = c.execute(q.get_tags)
         tags_in_db = []
         for each in res:
@@ -173,35 +208,47 @@ class Parser:
         non_existant_tags = []
         existant_tags = []
         tags = [a[1:] for a in tags]
+        for each in tags:
+            if each not in tags_in_db:
+                print(f'#{each} does not exist.')
+                non_existant_tags.append(each)
+                continue
+            existant_tags.append(each)
+        d(print, f'Non existant tags: {non_existant_tags}')
+        d(print, f'Existant tags: {existant_tags}')
         create_tags_script = ''
         for each in non_existant_tags:
+            # FIXME: extract this y/n to a function.
             ans = input(
                 f'Tag #{each} does not exist, do you want to create it?(y/n)')
-            ans = ans.tolower()
+            ans = ans.lower()
             if(ans == 'y' or ans == 'yes'):
-                create_tags_script = f'{create_tags_script}\nINSERT INTO tags(name,questions_count) VALUES ({each},0);'
+                # FIXME: move this to queries.py for safety :)
+                create_tags_script = f'{create_tags_script}\nINSERT INTO tags(name,questions_count) VALUES (\'{each}\',1);'
         c.executescript(create_tags_script)
-        s.conn.commit()
+        s['conn'].commit()
         tq_insert_script = ''
         for q_id in questions:
             for tag in tags:
                 tq_insert_script = f'{tq_insert_script}\nINSERT OR REPLACE INTO tq_relation(question_id,tag_id) VALUES ({q_id},(SELECT id FROM tags WHERE name=\'{tag}\'));'
         c.executescript(tq_insert_script)
-        s.conn.commit()
+        for tag in tags:
+            c.execute(q.update_questions_count, [each])
+        s['conn'].commit()
 
 
 def handle_parser_action():
     pa = constants.parser_actions
-    if s.parser_action == pa.LIST_TAGS:
+    if s['parser_action'] == pa.LIST_TAGS:
         Parser.list_tags()
         return
-    if s.parser_action == pa.OPEN_QUESTIONS:
+    if s['parser_action'] == pa.OPEN_QUESTIONS:
         Parser.open_questions()
         return
-    if s.parser_action == pa.LIST_QUESTIONS_OF_TAGS:
+    if s['parser_action'] == pa.LIST_QUESTIONS_OF_TAGS:
         Parser.list_questions_of_tags()
         return
-    if s.parser_action == pa.ADD_QUESTIONS_TO_TAGS:
+    if s['parser_action'] == pa.ADD_QUESTIONS_TO_TAGS:
         Parser.add_questions_to_tags()
         return
     invalid_command()
