@@ -9,7 +9,7 @@ import requests
 import json
 from gateoverflow.logger import d
 from gateoverflow import constants
-from gateoverflow import state as s
+from gateoverflow.state import state as s
 from gateoverflow import queries as q
 # TODO: oh poor me, please update this function later for avoiding embarrassement
 
@@ -114,7 +114,7 @@ def open_link(link):
 
 
 def uncrawled_metadata_count():
-    c = s.cursor
+    c = s['cursor']
     c.execute(q.uncrawled_metadata_count)
     res = c.fetchone()
     d(print, f'Unscraped Records: {res[0]}')
@@ -122,19 +122,23 @@ def uncrawled_metadata_count():
 
 
 def crawl_metadata():
-    c = s.cursor
+    c = s['cursor']
     c.execute(q.get_unscraped_question_ids)
     res = c.fetchall()
     stmt = ""
     for each in res:
         each = int(each[0])
         data = get_metadata(f'https://gateoverflow.in/{each}')
+        d(print, f'{data}')
+        d(print, f'questionid: {each}')
+        c.execute(q.update_crawl_attempts, [each])
         if data == None:
             continue
         c.execute(q.insert_into_metadata, (each, data['title'],
                                            data['desc'], data['image_url']))
         c.execute(
-            q.update_metadata_scraped_questions, (each))
+            q.update_metadata_scraped_questions, [each])
+    c.execute(q.delete_invalid_questions, [s['crawl_attempts_limit']])
 
 
 '''
@@ -155,9 +159,9 @@ def get_metadata(link):
         res['title'] = data['title']
         res['desc'] = data['description']
         res['image_url'] = data['image']
-        res['title'] = data['title']
-    except:
+    except Exception as e:
         print('maybe internet is down. Error, Skipping!')
+        d(print, f'Error: {e}')
         res = None
     return res
 
@@ -170,4 +174,11 @@ def prettify_table(data, headers):
 
 
 def print_logo():
-    print(f'{constants.colors.GREEN}{constants.colors.BOLD}{s.title_text}{constants.colors.END}{constants.colors.END}')
+    print(
+        f'{constants.colors.GREEN}{constants.colors.BOLD}{prettify_table([[s["title_text"].upper()]], [])}{constants.colors.END}{constants.colors.END}')
+    print(s['user'].greet())
+
+
+def ask():
+    q = input("Do you want to continue?(yes/no) (default: no): ")
+    return q.lower() == 'yes' or q.lower() == 'y'
